@@ -161,7 +161,7 @@ pub enum ContentInfo {
 
 impl ContentInfo {
     pub fn parse(r: BERReader) -> Result<Self, ASN1Error> {
-        Ok(r.read_sequence(|r| {
+        r.read_sequence(|r| {
             let content_type = r.next().read_oid()?;
             if content_type == *OID_DATA_CONTENT_TYPE {
                 let data = r.next().read_tagged(Tag::context(0), |r| r.read_bytes())?;
@@ -179,7 +179,7 @@ impl ContentInfo {
                 content_type,
                 content,
             }))
-        })?)
+        })
     }
     pub fn data(&self, password: &[u8]) -> Option<Vec<u8>> {
         match self {
@@ -357,7 +357,7 @@ pub struct MacData {
 
 impl MacData {
     pub fn parse(r: BERReader) -> Result<MacData, ASN1Error> {
-        Ok(r.read_sequence(|r| {
+        r.read_sequence(|r| {
             let mac = DigestInfo::parse(r.next())?;
             let salt = r.next().read_bytes()?;
             let iterations = r.next().read_u32()?;
@@ -366,7 +366,7 @@ impl MacData {
                 salt,
                 iterations,
             })
-        })?)
+        })
     }
 
     pub fn write(&self, w: DERWriter) {
@@ -444,7 +444,7 @@ impl PFX {
         let encrypted_data =
             pbe_with_sha_and3_key_triple_des_cbc_encrypt(key_der, &password, &salt, ITERATIONS)?;
         let param = Pkcs12PbeParams {
-            salt: salt,
+            salt,
             iterations: ITERATIONS,
         };
         let key_bag_inner = SafeBagKind::Pkcs8ShroudedKeyBag(EncryptedPrivateKeyInfo {
@@ -646,15 +646,24 @@ fn pbe_with_sha1_and40_bit_rc2_cbc(
     salt: &[u8],
     iterations: u64,
 ) -> Option<Vec<u8>> {
-    use block_modes::{block_padding::Pkcs7, BlockMode, Cbc};
+    use cbc::{
+        cipher::{block_padding::Pkcs7, BlockDecryptMut, KeyIvInit},
+        Decryptor,
+    };
     use rc2::Rc2;
-    type Rc2Cbc = Cbc<Rc2, Pkcs7>;
+    type Rc2Cbc = Decryptor<Rc2>;
 
     let dk = pbepkcs12sha1(password, salt, iterations, 1, 5);
     let iv = pbepkcs12sha1(password, salt, iterations, 2, 8);
 
     let rc2 = Rc2Cbc::new_from_slices(&dk, &iv).ok()?;
-    rc2.decrypt_vec(data).ok()
+    let mut buf = vec![0; data.len()];
+    let len = rc2
+        .decrypt_padded_b2b_mut::<Pkcs7>(data, &mut buf)
+        .ok()?
+        .len();
+    buf.resize(len, 0);
+    Some(buf)
 }
 
 fn pbe_with_sha1_and40_bit_rc2_cbc_encrypt(
@@ -663,15 +672,26 @@ fn pbe_with_sha1_and40_bit_rc2_cbc_encrypt(
     salt: &[u8],
     iterations: u64,
 ) -> Option<Vec<u8>> {
-    use block_modes::{block_padding::Pkcs7, BlockMode, Cbc};
+    use cbc::{
+        cipher::{block_padding::Pkcs7, BlockEncryptMut, BlockSizeUser, KeyIvInit},
+        Encryptor,
+    };
     use rc2::Rc2;
-    type Rc2Cbc = Cbc<Rc2, Pkcs7>;
+    type Rc2Cbc = Encryptor<Rc2>;
 
     let dk = pbepkcs12sha1(password, salt, iterations, 1, 5);
     let iv = pbepkcs12sha1(password, salt, iterations, 2, 8);
 
     let rc2 = Rc2Cbc::new_from_slices(&dk, &iv).ok()?;
-    Some(rc2.encrypt_vec(data))
+    let mut len = data.len();
+    let bs = Rc2::block_size();
+    let extra = len % bs;
+    if extra != 0 {
+        len += bs - extra;
+    }
+    let mut buf = vec![0; len];
+    rc2.encrypt_padded_b2b_mut::<Pkcs7>(data, &mut buf).unwrap();
+    Some(buf)
 }
 
 fn pbe_with_sha_and3_key_triple_des_cbc(
@@ -680,15 +700,24 @@ fn pbe_with_sha_and3_key_triple_des_cbc(
     salt: &[u8],
     iterations: u64,
 ) -> Option<Vec<u8>> {
-    use block_modes::{block_padding::Pkcs7, BlockMode, Cbc};
+    use cbc::{
+        cipher::{block_padding::Pkcs7, BlockDecryptMut, KeyIvInit},
+        Decryptor,
+    };
     use des::TdesEde3;
-    type TDesCbc = Cbc<TdesEde3, Pkcs7>;
+    type TDesCbc = Decryptor<TdesEde3>;
 
     let dk = pbepkcs12sha1(password, salt, iterations, 1, 24);
     let iv = pbepkcs12sha1(password, salt, iterations, 2, 8);
 
     let tdes = TDesCbc::new_from_slices(&dk, &iv).ok()?;
-    tdes.decrypt_vec(data).ok()
+    let mut buf = vec![0; data.len()];
+    let len = tdes
+        .decrypt_padded_b2b_mut::<Pkcs7>(data, &mut buf)
+        .ok()?
+        .len();
+    buf.resize(len, 0);
+    Some(buf)
 }
 
 fn pbe_with_sha_and3_key_triple_des_cbc_encrypt(
@@ -697,15 +726,27 @@ fn pbe_with_sha_and3_key_triple_des_cbc_encrypt(
     salt: &[u8],
     iterations: u64,
 ) -> Option<Vec<u8>> {
-    use block_modes::{block_padding::Pkcs7, BlockMode, Cbc};
+    use cbc::{
+        cipher::{block_padding::Pkcs7, BlockEncryptMut, BlockSizeUser, KeyIvInit},
+        Encryptor,
+    };
     use des::TdesEde3;
-    type TDesCbc = Cbc<TdesEde3, Pkcs7>;
+    type TDesCbc = Encryptor<TdesEde3>;
 
     let dk = pbepkcs12sha1(password, salt, iterations, 1, 24);
     let iv = pbepkcs12sha1(password, salt, iterations, 2, 8);
 
     let tdes = TDesCbc::new_from_slices(&dk, &iv).ok()?;
-    Some(tdes.encrypt_vec(data))
+    let mut len = data.len();
+    let bs = TdesEde3::block_size();
+    let extra = len % bs;
+    if extra != 0 {
+        len += bs - extra;
+    }
+    let mut buf = vec![0; len];
+    tdes.encrypt_padded_b2b_mut::<Pkcs7>(data, &mut buf)
+        .unwrap();
+    Some(buf)
 }
 
 fn bmp_string(s: &str) -> Vec<u8> {
@@ -800,7 +841,7 @@ fn test_encrypted_private_key_info() {
     let der = yasna::construct_der(|w| {
         epki.write(w);
     });
-    let epki2 = yasna::parse_ber(&der, |r| EncryptedPrivateKeyInfo::parse(r)).unwrap();
+    let epki2 = yasna::parse_ber(&der, EncryptedPrivateKeyInfo::parse).unwrap();
     assert_eq!(epki2, epki);
 }
 
@@ -919,13 +960,13 @@ impl PKCS12Attribute {
             }
             PKCS12Attribute::LocalKeyId(id) => {
                 w.next().write_oid(&OID_LOCAL_KEY_ID);
-                w.next().write_set_of(|w| w.next().write_bytes(&id))
+                w.next().write_set_of(|w| w.next().write_bytes(id))
             }
             PKCS12Attribute::Other(other) => {
                 w.next().write_oid(&other.oid);
                 w.next().write_set_of(|w| {
                     for bytes in other.data.iter() {
-                        w.next().write_der(&bytes);
+                        w.next().write_der(bytes);
                     }
                 })
             }
@@ -949,7 +990,7 @@ impl SafeBag {
 
             let attributes = r
                 .read_optional(|r| r.collect_set_of(PKCS12Attribute::parse))?
-                .unwrap_or_else(|| vec![]);
+                .unwrap_or_else(Vec::new);
 
             Ok(SafeBag { bag, attributes })
         })
